package com.simuwang.manage.api.company;

import com.simuwang.base.common.support.MybatisPage;
import com.simuwang.base.pojo.dto.query.CompanyEmailHistoryPageQuery;
import com.simuwang.base.pojo.dto.query.CompanyEmailPageQuery;
import com.simuwang.base.pojo.vo.*;
import com.simuwang.logging.SystemLog;
import com.simuwang.manage.service.CompanyEmailConfigService;
import com.simuwang.manage.service.CompanyEmailSendHistoryService;
import com.simuwang.manage.service.CompanyReportFrequencyService;
import com.simuwang.manage.service.ContactInformationService;
import com.smppw.common.pojo.ResultVo;
import com.smppw.common.pojo.enums.status.ResultCode;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;
/**
 * 公司邮箱配置管理
 * Author:   chenjianhua
 * Date:     2024/9/6 15:25
 * Description: ${DESCRIPTION}
 */
@SystemLog(value = "公司邮箱配置管理")
@RestController
@RequestMapping("/v1/company")
public class CompanyContactInformationController{
    private static final Logger logger = LoggerFactory.getLogger(CompanyContactInformationController.class);
    @Autowired
    private CompanyEmailSendHistoryService companyEmailSendHistoryService;

    @Autowired
    private CompanyEmailConfigService companyEmailConfigService;

    @Autowired
    private CompanyReportFrequencyService companyReportFrequencyService;

    @Autowired
    private ContactInformationService contactInformationService;
    /**
     * 公司邮箱页面展示查询
     * @param companyEmailPageQuery
     * @return
     */
    @SystemLog(value = "公司邮箱查询")
    @RequestMapping("search-company-email")
    public MybatisPage<CompanyEmailPageVO> searchCompanyEmail(CompanyEmailPageQuery companyEmailPageQuery){
        MybatisPage<CompanyEmailPageVO> result = companyEmailSendHistoryService.searchCompanyEmail(companyEmailPageQuery);
        return result;
    }

    /**
     * 根据公司ID查询邮箱发送历史
     * @param  companyEmailHistoryPageQuery
     * @return
     */
    @SystemLog(value = "公司邮箱发送历史查询")
    @RequestMapping("search-email-history")
    public MybatisPage<CompanyEmailSendHistoryVO> searchEmailHistory(CompanyEmailHistoryPageQuery companyEmailHistoryPageQuery){
        MybatisPage<CompanyEmailSendHistoryVO> result = companyEmailSendHistoryService.searchEmailHistory(companyEmailHistoryPageQuery);
        return result;
    }

    /**
     * 根据公司ID批量删除邮箱配置
     * @param idListVO
     * @return
     */
    @SystemLog(value = "删除公司邮箱配置", type = SystemLog.Type.DELETE)
    @PostMapping("delete-company-email")
    public ResultVo deleteEmail(@RequestBody IdListVO idListVO){
        ResultVo vo = companyEmailSendHistoryService.deleteEmail(idListVO.getIdList());
        return vo;
    }

    /**
     * 根据ID批量删除邮箱发送历史数据
     * @param comyanyEmailIdVO 已选行的idList
     * @return
     */
    @SystemLog(value = "删除公司邮箱发送历史", type = SystemLog.Type.DELETE)
    @PostMapping("delete-email-history")
    public boolean deleteEmailHistory(@RequestBody IdListVO comyanyEmailIdVO){
        companyEmailSendHistoryService.deleteEmailHistory(comyanyEmailIdVO.getIdList());
        return true;
    }

    /**
     * 保存邮箱配置
     * @param contactInformationVO
     * @return
     */
    @SystemLog(value = "保存公司邮箱", type = SystemLog.Type.INSERT)
    @PostMapping("save-email-config")
    public ResultVo saveContactInformation(@RequestBody List<ContactInformationVO> contactInformationVO){
        ResultVo vo = new ResultVo(ResultCode.SAVE_SUCCESS.getCode());
        try{
            vo = contactInformationService.saveContactInformation(contactInformationVO);
        }catch (Exception e){
            logger.error(e.getMessage(),e);
            vo.setData(false);
            vo.setCode(ResultCode.SAVE_FAILED.getCode());
        }
        return vo;
    }

    /**
     * 保存公司数据报送频率
     * @param companyReportFrequencyVO
     * @return
     */
    @SystemLog(value = "保存公司数据报送频率", type = SystemLog.Type.INSERT)
    @PostMapping("/save-company-frequency")
    public ResultVo saveCompanyEmailConfig(@RequestBody CompanyReportFrequencyVO companyReportFrequencyVO){
        ResultVo vo = companyReportFrequencyService.saveCompanyReportFrequency(companyReportFrequencyVO);
        return vo;
    }


    @SystemLog(value = "查询公司数据报送频率")
    @RequestMapping("get-company-frequency")
    public ResultVo searchCompanyFrequency(CompanyIdVO companyIdVO){
        ResultVo result = companyReportFrequencyService.searchCompanyFrequency(companyIdVO);
        return result;
    }
    /**
     * 根据companyIdList发送数据缺失邮件
     * @param idListVO  已选行的companyIdList
     * @return
     */
    @SystemLog(value = "发送净值缺失数据")
    @PostMapping("send-company-email")
    public ResultVo sendCompanyEmail(@RequestBody IdListVO idListVO){
        ResultVo vo = new ResultVo(ResultCode.SUCCESS.getCode());
        companyEmailConfigService.sendCompanyEmail(idListVO.getIdList());
        vo.setData(true);
        vo.setMsg("发送中，请稍后");
        return vo;
    }

    /**
     * 根据companyIdList发送数据缺失邮件
     * @param idListVO  已选行的companyIdList
     * @return
     */
    @SystemLog(value = "发送分红缺失数据")
    @PostMapping("send-distribute-email")
    public boolean sendDistributeEmail(@RequestBody IdListVO idListVO){
        companyEmailConfigService.sendDistributeEmailById(idListVO.getIdList());
        return true;
    }

    /**
     * 上传联系人信息
     *
     * @param file
     * @return
     */
    @SystemLog(value = "上传联系人信息", type = SystemLog.Type.UPLOAD_OR_IMPORT)
    @PostMapping("upload-contact-list")
    public ResultVo uploadContactInformation(@RequestPart(value = "file") MultipartFile file) {
        ResultVo vo = contactInformationService.uploadContactInformation(file);
        return vo;
    }

    /**
     * 上传联系人信息
     *
     * @param files
     * @return
     */
    @SystemLog(value = "发送邮件", type = SystemLog.Type.UPLOAD_OR_IMPORT)
    @PostMapping("send-email")
    public ResultVo sendEmail(@RequestPart(value = "files",required = false) List<MultipartFile> files,@RequestParam("contractId") String contractId,@RequestParam("emailTitle") String emailTitle,
                              @RequestParam("emailBody") String emailBody,@RequestParam("templateCode") String templateCode) {
        ResultVo vo = contactInformationService.sendEmail(files,contractId,emailTitle,templateCode,emailBody);
        return vo;
    }

}
